import { NgModule, EventEmitter, Component, ViewEncapsulation, ContentChild, Input, Output, Injectable } from '@angular/core';
import { CommonModule } from '@angular/common';
import { HammerGestureConfig, HammerModule, HAMMER_GESTURE_CONFIG } from '@angular/platform-browser';
import { MatRippleModule } from '@angular/material/core';
import { MatIconModule } from '@angular/material/icon';
import { MatListModule } from '@angular/material/list';
import { trigger, transition, style, sequence, animate, keyframes } from '@angular/animations';
import { DIRECTION_HORIZONTAL } from 'hammerjs';
import { FlexLayoutModule } from '@angular/flex-layout';

class MaterialImportModule {
}
MaterialImportModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    MatIconModule,
                    MatListModule,
                    MatRippleModule
                ],
                declarations: [],
                exports: [
                    MatIconModule,
                    MatListModule,
                    MatRippleModule
                ]
            },] }
];

const Constants = {
    MAX_OFFSET: 500,
    MIN_OFFSET: 0,
    DEFAULT_THRESHOLD: 100,
    DEFAULT_LIMIT: 300,
};
var Warnings;
(function (Warnings) {
    Warnings["ADDING_DEFAULT_SLIDE_THRESHOLD"] = "Will keep it default i.e.";
    Warnings["SLIDE_THRESHOLD_NOT_FOUND"] = "You have not provided the slideThreshold.";
    Warnings["ZERO_SLIDE_THRESHOLD_NOT_ALLOWED"] = "slideThreshold value can not be 0 or less than 0.";
    Warnings["MAX_OFFSET_EXCEEDED"] = "Swipe action threshold value should be less than 50.";
    Warnings["LIMIT_TOO_LOW"] = "Swipe limit should exceed threshold.";
    Warnings["INVALID_SLIDE_THRESHOLD_NOT_ALLOWED"] = "slideThreshold value is invalid, Expecting number between 0 to 50.";
})(Warnings || (Warnings = {}));

const offset = '50';
const rowsAnimation = trigger('rowsAnimation', [
    transition('void => *', [
        style({
            height: '*',
            opacity: '0',
            transform: 'translateY(' + (-offset).toString() + 'px)',
            'box-shadow': 'none'
        }),
        sequence([
            animate('.2s ease', style({
                height: '*',
                opacity: '.2',
                transform: 'translateX(0)',
                'box-shadow': 'none'
            })),
            animate('.35s ease', style({ height: '*', opacity: 1, transform: 'translateX(0)' }))
        ])
    ]),
    transition('* => void', [
        style({ height: '*', opacity: '1', transform: 'translateX(0px) translateY(' + (-offset).toString() + 'px)', 'box-shadow': 'none' }),
        sequence([
            animate('.2s ease', style({
                height: '*',
                opacity: 0,
                transform: 'translateX(' + (-offset).toString() + 'px) translateY(' + (-offset).toString() + 'px)'
            }))
        ])
    ]),
]);

class MatListTouchComponent {
    constructor() {
        this.swipeLeftAction = new EventEmitter();
        this.swipeRightAction = new EventEmitter();
        this.tapAction = new EventEmitter();
        this.leftColor = 'green';
        this.leftIcon = 'check';
        this.rightColor = 'red';
        this.rightIcon = 'not_interested';
        this.defaultSwipeColor = 'gray';
        this.swipeThreshold = Constants.DEFAULT_THRESHOLD;
        this.swipeLimit = Constants.DEFAULT_LIMIT;
        this.silenceWarnings = false;
        this.currentSwipeColor = this.defaultSwipeColor;
        this.lastAnimatedIndex = null;
    }
    ngOnInit() {
        this.resetSwipeList();
    }
    evalLeftBorder(item, index) {
        if (this.leftBorderEval) {
            return this.leftBorderEval(item, index);
        }
        return '';
    }
    evalRightBorder(item, index) {
        if (this.rightBorderEval) {
            return this.rightBorderEval(item, index);
        }
        return '';
    }
    resetSwipeList() {
        this.setThreshold();
    }
    setThreshold() {
        if (this.swipeThreshold < Constants.MIN_OFFSET || this.swipeThreshold > Constants.MAX_OFFSET) {
            if (this.swipeThreshold > Constants.MAX_OFFSET) {
                this.logWarnings(Warnings.MAX_OFFSET_EXCEEDED, `${Warnings.ADDING_DEFAULT_SLIDE_THRESHOLD} ${Constants.DEFAULT_THRESHOLD}%.`);
            }
            if (this.swipeThreshold < Constants.MIN_OFFSET || this.swipeThreshold === Constants.MIN_OFFSET) {
                this.logWarnings(Warnings.ZERO_SLIDE_THRESHOLD_NOT_ALLOWED, `${Warnings.ADDING_DEFAULT_SLIDE_THRESHOLD} ${Constants.DEFAULT_THRESHOLD}%.`);
            }
            this.swipeThreshold = Constants.DEFAULT_THRESHOLD;
        }
        if (this.swipeLimit < this.swipeThreshold) {
            this.swipeLimit = 100 - this.swipeThreshold;
            this.logWarnings(Warnings.LIMIT_TOO_LOW);
        }
    }
    panMoveEvent(action, elementRef, item, index) {
        if (this.disableActionsEval && this.disableActionsEval(item, index)) {
            elementRef.style.left = 0;
            return;
        }
        if (!this.swipeLeftAction.observers.length && !this.swipeRightAction.observers.length) {
            elementRef.style.left = 0;
            return;
        }
        // DEBUG THIS HACK
        if (Math.abs(action.deltaY) > 50) {
            elementRef.style.left = 0;
            return;
        }
        const offset = action.deltaX;
        if (Math.abs(action.deltaX) < this.swipeLimit) {
            elementRef.style.left = action.deltaX + 'px';
        }
        else {
            if (action.deltaX > 0) {
                elementRef.style.left = this.swipeLimit + 'px';
            }
            else {
                elementRef.style.left = -this.swipeLimit + 'px';
            }
        }
        if (offset > this.swipeThreshold) {
            this.currentSwipeColor = this.leftColor;
        }
        else if (offset < -this.swipeThreshold) {
            this.currentSwipeColor = this.rightColor;
        }
        else {
            this.currentSwipeColor = this.defaultSwipeColor;
        }
    }
    panEndEvent(event, item, index) {
        if (event.deltaX > this.swipeThreshold) {
            this.emitLeftAction(index);
        }
        else if (event.deltaX < -this.swipeThreshold) {
            this.emitRightAction(index);
        }
        this.lastAnimatedIndex = index;
    }
    tapEvent(index) {
        const tapItem = this.dataSource[index];
        if (this.tapAction.observers) {
            this.tapAction.emit({
                index,
                data: tapItem
            });
        }
    }
    slideComplete(event) {
        event.element.style.left = '0px';
        this.lastAnimatedIndex = null;
    }
    emitLeftAction(index) {
        const actionItem = this.dataSource[index];
        if (this.swipeLeftAction.observers) {
            this.swipeLeftAction.emit({
                index,
                data: actionItem
            });
        }
    }
    emitRightAction(index) {
        const actionItem = this.dataSource[index];
        if (this.swipeRightAction.observers) {
            this.swipeRightAction.emit({
                index,
                data: actionItem
            });
        }
    }
    logWarnings(warningFor, extraMessage) {
        if (!this.silenceWarnings) {
            switch (warningFor) {
                case Warnings.SLIDE_THRESHOLD_NOT_FOUND:
                case Warnings.ZERO_SLIDE_THRESHOLD_NOT_ALLOWED:
                case Warnings.MAX_OFFSET_EXCEEDED:
                case Warnings.INVALID_SLIDE_THRESHOLD_NOT_ALLOWED:
                    extraMessage ?
                        console.warn(this.getConstValue(warningFor)) :
                        console.warn(this.getConstValue(warningFor), extraMessage);
                    break;
            }
        }
    }
    getConstValue(constantName) {
        return Warnings[constantName];
    }
}
MatListTouchComponent.decorators = [
    { type: Component, args: [{
                selector: 'mat-list-touch',
                template: "<mat-list *ngIf=\"dataSource\" class=\"list-canvas\" ngClass.gt-sm=\"list-canvas-big\">\n    <ng-container *ngFor=\"let item of dataSource; let i = index;\">\n        <mat-list-item class=\"separator\" *ngIf=\"separatorEval(dataSource, i)\">\n            <div fxLayout=\"column\" class=\"list-item-container stripe-angled\">\n                <ng-template ngFor let-item [ngForOf]=\"[item]\" [ngForTemplate]=\"separatorTemplate\">\n                </ng-template>\n            </div>\n        </mat-list-item>\n        <mat-list-item [@rowsAnimation]=\"\">\n            <div class=\"slide-action-indicator\" [ngStyle]=\"{'background-color': currentSwipeColor}\">\n                <i class=\"material-icons slide-action-icon\" *ngIf=\"swipeLeftAction.observers\">{{leftIcon}}</i>\n                <span>&nbsp;</span>\n                <i class=\"material-icons slide-action-icon\" *ngIf=\"swipeRightAction.observers\">{{rightIcon}}</i>\n            </div>\n            <div fxLayout=\"column\" style=\"width:100%; height:100%;\">\n                <div #element fxLayout=\"row\"\n                    [@slideRow]=\"lastAnimatedIndex === i\"\n                    (@slideRow.done)=slideComplete($event) \n                    (panend)=\"panEndEvent($event, element, i)\"\n                    (panleft)=\"panMoveEvent($event, element, item, i)\"\n                    (panright)=\"panMoveEvent($event, element, item, i)\" \n                    (tap)=\"tapEvent(i)\"\n                    class=\"list-item-container list-item-hoverable\"\n                    [ngStyle]=\"{'border-left': evalLeftBorder(item, i), 'border-right': evalRightBorder(item, i)}\">\n                    <div class=\"list-item-identity\">\n                        <ng-template ngFor let-item [ngForOf]=\"[item]\" [ngForTemplate]=\"identityTemplate\">\n                        </ng-template>\n                    </div>\n                    <ng-template ngFor let-item [ngForOf]=\"[item]\" [ngForTemplate]=\"contentTemplate\">\n                    </ng-template>\n                </div>\n                <mat-divider *ngIf=\"i !== (dataSource.length - 1)\"></mat-divider>\n            </div>\n        </mat-list-item>\n    </ng-container>\n</mat-list>\n",
                encapsulation: ViewEncapsulation.None,
                animations: [
                    trigger('slideRow', [
                        transition('* => *', animate(100, keyframes([
                            style({ left: '*', offset: 0 }),
                            style({ left: '0', offset: 1 }),
                        ])))
                    ]),
                    rowsAnimation,
                ],
                styles: [".list-canvas.mat-list-base{padding-top:0}.stripe-angled{background:repeating-linear-gradient(45deg,rgba(33,150,243,.1),rgba(33,150,243,.1) 5px,#f1f8ff 10px,#f1f8ff 20px);color:#000}.list-canvas mat-list-item{height:60px!important}.list-canvas mat-list-item .mat-list-item-content{overflow:hidden;padding:0!important}.list-canvas mat-list-item.separator{height:20px!important}.list-canvas .slide-action-indicator{align-items:center;background-color:green;display:flex;flex-shrink:0;height:100%;justify-content:space-between;position:absolute;width:100%}.list-canvas .slide-action-indicator .slide-action-icon{color:#f1f8ff;font-size:24px;height:24px;margin-left:16px;margin-right:16px;width:24px}.list-canvas .list-item-container{align-items:center;background-color:#fff;display:flex;flex-direction:row;height:inherit;position:relative;width:100%}.list-canvas .list-item-container>.list-item-identity{color:rgba(0,0,0,.8);padding-left:16px!important;padding-right:16px!important}.list-canvas .list-item-container>.list-item-details{padding-right:16px!important}.list-canvas-big mat-list-item{height:80px!important}.list-canvas-big .list-item-container>.list-item-identity{padding-left:24px!important;padding-right:24px!important}.list-canvas-big .list-item-container>.list-item-details{padding-right:24px!important}.list-item-hoverable:hover{background-color:#f0f8ff}"]
            },] }
];
MatListTouchComponent.propDecorators = {
    contentTemplate: [{ type: ContentChild, args: ['contentTemplate', { static: true },] }],
    separatorTemplate: [{ type: ContentChild, args: ['separatorTemplate', { static: true },] }],
    identityTemplate: [{ type: ContentChild, args: ['identityTemplate', { static: true },] }],
    dataSource: [{ type: Input }],
    swipeLeftAction: [{ type: Output }],
    swipeRightAction: [{ type: Output }],
    tapAction: [{ type: Output }],
    separatorEval: [{ type: Input }],
    leftBorderEval: [{ type: Input }],
    rightBorderEval: [{ type: Input }],
    disableActionsEval: [{ type: Input }],
    leftColor: [{ type: Input }],
    leftIcon: [{ type: Input }],
    rightColor: [{ type: Input }],
    rightIcon: [{ type: Input }],
    defaultSwipeColor: [{ type: Input }],
    swipeThreshold: [{ type: Input }],
    swipeLimit: [{ type: Input }],
    silenceWarnings: [{ type: Input }]
};

class GestureConfig extends HammerGestureConfig {
    constructor() {
        super(...arguments);
        this.overrides = {
            pan: {
                enable: true,
                touchAction: 'auto',
                direction: DIRECTION_HORIZONTAL
            },
            pinch: {
                enable: false
            },
            rotate: {
                enable: false
            },
            swipe: {
                enable: false
            }
        };
    }
}
GestureConfig.decorators = [
    { type: Injectable }
];

class MatListTouchModule {
}
MatListTouchModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    MaterialImportModule,
                    FlexLayoutModule,
                    HammerModule
                ],
                declarations: [MatListTouchComponent],
                exports: [
                    MatListTouchComponent
                ],
                providers: [{
                        provide: HAMMER_GESTURE_CONFIG,
                        useClass: GestureConfig,
                    }]
            },] }
];

/*
 * Public API Surface of mat-list-touch
 */

/**
 * Generated bundle index. Do not edit.
 */

export { GestureConfig, MatListTouchModule, MaterialImportModule as ɵa, MatListTouchComponent as ɵb, rowsAnimation as ɵc };
//# sourceMappingURL=mat-list-touch.js.map
